import logging

from vortex.handler.TupleDataObservableHandler import TupleDataObservableHandler

from peek_plugin_inbox._private.PluginNames import inboxFilt, \
    inboxObservableName
from peek_plugin_inbox._private.server.tuple_providers.ActivityTupleProvider import \
    ActivityTupleProvider
from peek_plugin_inbox._private.storage.Activity import Activity
from peek_plugin_inbox._private.storage.Task import Task
from .tuple_providers.TaskTupleProvider import TaskTupleProvider

logger = logging.getLogger(__name__)


def makeTupleDataObservableHandler(ormSessionCreator):
    observable = TupleDataObservableHandler(observableName=inboxObservableName,
                                            additionalFilt=inboxFilt)

    observable.addTupleProvider(Task.tupleName(), TaskTupleProvider(ormSessionCreator))

    observable.addTupleProvider(Activity.tupleName(),
                                ActivityTupleProvider(ormSessionCreator))
    return observable



from peek_plugin_base.PeekVortexUtil import peekServerName
from peek_plugin_inbox._private.PluginNames import inboxFilt, \
    inboxObservableName
from vortex.handler.TupleDataObservableProxyHandler import TupleDataObservableProxyHandler


def makeTupleDataObservableProxy():
    return TupleDataObservableProxyHandler(observableName=inboxObservableName,
                                           proxyToVortexName=peekServerName,
                                           additionalFilt=inboxFilt)




"""Initial table creation

Peek Plugin Database Migration Script

Revision ID: db5936c4d50d
Revises: 
Create Date: 2017-03-06 21:33:18.771121

"""

# revision identifiers, used by Alembic.
revision = 'db5936c4d50d'
down_revision = None
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('Activity',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('uniqueId', sa.String(length=100), nullable=False),
    sa.Column('userId', sa.String(length=50), nullable=False),
    sa.Column('dateTime', sa.DateTime(), nullable=False),
    sa.Column('title', sa.String(length=100), nullable=False),
    sa.Column('description', sa.String(length=2000), nullable=True),
    sa.Column('iconPath', sa.String(length=200), nullable=True),
    sa.Column('routePath', sa.String(length=200), nullable=True),
    sa.Column('routeParamJson', sa.String(length=200), nullable=True),
    sa.Column('autoDeleteDateTime', sa.DateTime(), nullable=False),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('uniqueId'),
    schema='pl_inbox'
    )
    op.create_table('Setting',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(), nullable=True),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_inbox'
    )
    op.create_table('Task',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('uniqueId', sa.String(length=100), nullable=False),
    sa.Column('userId', sa.String(length=50), nullable=False),
    sa.Column('dateTime', sa.DateTime(), nullable=False),
    sa.Column('title', sa.String(length=100), nullable=False),
    sa.Column('description', sa.String(length=2000), nullable=True),
    sa.Column('iconPath', sa.String(length=200), nullable=True),
    sa.Column('routePath', sa.String(length=200), nullable=True),
    sa.Column('routeParamJson', sa.String(length=200), nullable=True),
    sa.Column('onDeliveredPayload', sa.LargeBinary(), nullable=True),
    sa.Column('onCompletedPayload', sa.LargeBinary(), nullable=True),
    sa.Column('onDeletedPayload', sa.LargeBinary(), nullable=True),
    sa.Column('autoComplete', sa.Integer(), server_default='0', nullable=False),
    sa.Column('autoDelete', sa.Integer(), server_default='0', nullable=False),
    sa.Column('stateFlags', sa.Integer(), server_default='0', nullable=False),
    sa.Column('notificationRequiredFlags', sa.Integer(), server_default='0', nullable=False),
    sa.Column('notificationSentFlags', sa.Integer(), server_default='0', nullable=False),
    sa.Column('displayAs', sa.Integer(), server_default='0', nullable=False),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('uniqueId'),
    schema='pl_inbox'
    )
    op.create_table('SettingProperty',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('settingId', sa.Integer(), nullable=False),
    sa.Column('key', sa.String(length=50), nullable=False),
    sa.Column('type', sa.String(length=16), nullable=True),
    sa.Column('int_value', sa.Integer(), nullable=True),
    sa.Column('char_value', sa.String(), nullable=True),
    sa.Column('boolean_value', sa.Boolean(), nullable=True),
    sa.ForeignKeyConstraint(['settingId'], ['pl_inbox.Setting.id'], ),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_inbox'
    )
    op.create_index('idx_SettingProperty_settingId', 'SettingProperty', ['settingId'], unique=False, schema='pl_inbox')
    op.create_table('TaskAction',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('taskId', sa.Integer(), nullable=False),
    sa.Column('title', sa.String(length=50), nullable=True),
    sa.Column('confirmMessage', sa.String(length=200), nullable=True),
    sa.Column('onActionPayload', sa.LargeBinary(), nullable=True),
    sa.ForeignKeyConstraint(['taskId'], ['pl_inbox.Task.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id'),
    schema='pl_inbox'
    )
    op.create_index('idx_TaskAction_taskId', 'TaskAction', ['taskId'], unique=False, schema='pl_inbox')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index('idx_TaskAction_taskId', table_name='TaskAction', schema='pl_inbox')
    op.drop_table('TaskAction', schema='pl_inbox')
    op.drop_index('idx_SettingProperty_settingId', table_name='SettingProperty', schema='pl_inbox')
    op.drop_table('SettingProperty', schema='pl_inbox')
    op.drop_table('Task', schema='pl_inbox')
    op.drop_table('Setting', schema='pl_inbox')
    op.drop_table('Activity', schema='pl_inbox')
    # ### end Alembic commands ###



"""Added onDialogConfirmPayload

Peek Plugin Database Migration Script

Revision ID: 543ec701c9f7
Revises: da155db8b8f1
Create Date: 2017-04-17 09:07:21.947217

"""

# revision identifiers, used by Alembic.
revision = '543ec701c9f7'
down_revision = 'da155db8b8f1'
branch_labels = None
depends_on = None

import sqlalchemy as sa
from alembic import op


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('Task',
                  sa.Column('onDialogConfirmPayload', sa.LargeBinary(), nullable=True),
                  schema='pl_inbox')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('Task', 'onDialogConfirmPayload', schema='pl_inbox')
    # ### end Alembic commands ###



"""timezone aware

Peek Plugin Database Migration Script

Revision ID: 2119b6aab2c5
Revises: 0f8cf37f3af3
Create Date: 2017-12-31 14:29:39.875909

"""

# revision identifiers, used by Alembic.
revision = '2119b6aab2c5'
down_revision = '0f8cf37f3af3'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # pl_inbox."Task"
    op.execute('''ALTER TABLE pl_inbox."Task"
                  ALTER COLUMN "dateTime" TYPE TIMESTAMP WITH TIME ZONE
                  USING "dateTime" AT TIME ZONE 'UTC'
               ''')

    op.execute('''ALTER TABLE pl_inbox."Task"
                  ALTER COLUMN "autoDeleteDateTime" TYPE TIMESTAMP WITH TIME ZONE
                  USING "autoDeleteDateTime" AT TIME ZONE 'UTC'
               ''')

    # pl_inbox."Activity"
    op.execute('''ALTER TABLE pl_inbox."Activity"
                  ALTER COLUMN "dateTime" TYPE TIMESTAMP WITH TIME ZONE
                  USING "dateTime" AT TIME ZONE 'UTC'
               ''')

    op.execute('''ALTER TABLE pl_inbox."Activity"
                  ALTER COLUMN "autoDeleteDateTime" TYPE TIMESTAMP WITH TIME ZONE
                  USING "autoDeleteDateTime" AT TIME ZONE 'UTC'
               ''')


def downgrade():
    pass



"""fix task priorities

Peek Plugin Database Migration Script

Revision ID: 36a9bc546606
Revises: 543ec701c9f7
Create Date: 2017-04-18 09:52:20.935251

"""

# revision identifiers, used by Alembic.
revision = '36a9bc546606'
down_revision = '543ec701c9f7'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    op.execute('''UPDATE "pl_inbox"."Task"
                  SET "displayPriority" = 1
                  WHERE "displayPriority" = 0;''')


def downgrade():
    pass



import logging

from peek_plugin_inbox._private.PluginNames import inboxFilt, \
    inboxActionProcessorName
from peek_plugin_inbox._private.server.controller.MainController import \
    MainController
from vortex.handler.TupleActionProcessor import TupleActionProcessor

logger = logging.getLogger(__name__)


def makeTupleActionProcessorHandler(inboxProcessor: MainController):
    processor = TupleActionProcessor(
        tupleActionProcessorName=inboxActionProcessorName,
        additionalFilt=inboxFilt,
        defaultDelegate=inboxProcessor)
    return processor
